package com.fzm.common.exception.handler;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpStatus;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.utils.JwtUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ：tangtuo
 * @date ：2021/04/30 16:35
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    @Resource
    private HttpServletResponse response;

    @ExceptionHandler(value = GlobalException.class)
    public ResponseModel<String> handlerGlobalException(GlobalException globalException) {
        log.error(globalException.getMessage(), globalException);
        response.setStatus(HttpStatus.HTTP_INTERNAL_ERROR);
        return ResponseModel.fail(globalException.getErrorCode(), globalException.getMessage());
    }

    @ExceptionHandler(value = Exception.class)
    public ResponseModel<String> handlerException(Exception exception) {
        log.error(exception.getMessage(), exception);
        response.setStatus(HttpStatus.HTTP_INTERNAL_ERROR);
        return ResponseModel.fail(exception);
    }

    @ExceptionHandler(value = NotLoginException.class)
    public ResponseModel<String> handlerNotLoginException(NotLoginException exception) {
        log.error(exception.getMessage(), exception);
        response.setStatus(HttpStatus.HTTP_INTERNAL_ERROR);
        if (exception.getMessage().contains("token已被顶下线")) {
            // 获取已过期的token
            String expireToken = StpUtil.getTokenValue();
            // 从已过期的token中获取用户id
            Integer userId = JwtUtil.getUserIdFromToken(expireToken);
            // 获取最新的token
            String tokenValue = StpUtil.getTokenValueByLoginId(userId);
            Date createTime = JwtUtil.getCreateTimeFromToken(tokenValue);
            String format = DateUtil.format(createTime, "yyyy-MM-dd HH:mm:ss");
            return ResponseModel.fail(ResultCode.UNAUTHORIZED,
                    String.format("您的账号于 %s 登录另外一台设备,如非您本人所为,请立即修改密码", format));

        }
        return ResponseModel.fail(exception);
    }

    @ExceptionHandler(value = {MethodArgumentNotValidException.class})
    public ResponseModel<String> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
        log.error(e.getMessage(), e);
        response.setStatus(HttpStatus.HTTP_INTERNAL_ERROR);
        List<ObjectError> allErrors = e.getBindingResult().getAllErrors();
        String errorMsg = allErrors.
                stream().
                map(DefaultMessageSourceResolvable::getDefaultMessage).
                collect(Collectors.joining("、"));
        return ResponseModel.fail(ResultCode.VALIDATE_FAILED, errorMsg);
    }
}
